package com.j_spaces.examples.benchmark;

import com.gigaspaces.internal.version.PlatformVersion;

/**
 * <pre>
 * Versions are denoted using a standard triplet of integers: MAJOR.MINOR.PATCH.
 * The basic intent is that:
 * <li>MAJOR versions are sync with GigaSpaces API version. </li>
 * <li>MINOR versions are incremented upon enhancements/features in Benchmark functionality.</li>
 * <li>PATCH level are incremented on each patch to the Benchmark functionality.</li>
 *
 * The major version number identifies the GigaSpaces API version.
 * Increments are based on the Platform version.
 *
 * The minor version number identifies enhancements/features to Benchmark functionality.
 * Increments to minor version set the patch version to zero.
 *
 * The patch version number identifies revisions.
 * Increments to patch number only.
 */
public final class BenchmarkVersion {
    private static final char SEP = '.';
    private static final String MAJOR = PlatformVersion.getVersion() +
            "-" + PlatformVersion.getMilestone() +
            "-" + PlatformVersion.getBuildNumber();

    /**
     * MINOR.PATCH
     */
    private static final String MINOR = "0";
    private static final String PATCH = "2";


    /**
     * only benchmark version
     */
    public static final String VERSION = MINOR + SEP + PATCH;

    /**
     * platform + benchmark version
     */
    public static final String FULL_VERSION = MAJOR + " " + VERSION;

}
